const status = async (...args) => {
  const [pool, readModelName, eventstoreAdapter, includeRuntimeStatus] = args;

  try {
    var _result;

    pool.activePassthrough = true;
    let result = null;
    let cursorAndEventTypes = [null, ['*']];
    const {
      PassthroughError,
      inlineLedgerRunQuery,
      schemaName,
      tablePrefix,
      escapeId,
      escapeStr
    } = pool;
    const databaseNameAsId = escapeId(schemaName);
    const databaseNameAsStr = escapeStr(schemaName);
    const ledgerTableNameAsId = escapeId(`${tablePrefix}__${schemaName}__LEDGER__`);
    const trxTableNameAsId = escapeId(`${tablePrefix}__${schemaName}__TRX__`);
    const ledgerTableNameAsStr = escapeStr(`${tablePrefix}__${schemaName}__LEDGER__`);
    const rows = await inlineLedgerRunQuery(`SELECT * FROM ${databaseNameAsId}.${ledgerTableNameAsId}
     WHERE "EventSubscriber" = ${escapeStr(readModelName)}
    `);

    if (rows.length === 1) {
      result = {
        eventSubscriber: readModelName,
        deliveryStrategy: 'inline-ledger',
        successEvent: rows[0].SuccessEvent,
        failedEvent: rows[0].FailedEvent,
        errors: rows[0].Errors,
        cursor: rows[0].Cursor,
        status: 'deliver'
      };

      if (result.errors != null) {
        result.status = 'error';
      } else if (rows[0].IsPaused) {
        result.status = 'skip';
      }

      cursorAndEventTypes = [result.cursor, rows[0].EventTypes != null ? rows[0].EventTypes : ['*']];
    }

    if (includeRuntimeStatus && ((_result = result) === null || _result === void 0 ? void 0 : _result.status) === 'deliver') {
      var _eventstoreAdapter$ge;

      let isActive = false;
      const endTime = Date.now() + 5000;

      for (let currentTime = 0; currentTime < endTime; currentTime = Date.now()) {
        try {
          var _rows$;

          const rows = await inlineLedgerRunQuery(`SELECT Count(*) AS "ActiveLocksCount" FROM pg_locks
            WHERE database = (SELECT oid FROM pg_database WHERE datname = current_database())
            AND relation = (SELECT oid FROM pg_class WHERE relname = ${ledgerTableNameAsStr}
            AND relnamespace = (SELECT oid FROM pg_namespace WHERE nspname = ${databaseNameAsStr}))
            AND pid = (
              SELECT CASE WHEN "T"."XaValue" IS NOT NULL THEN CAST("T"."XaValue" AS INT)
              ELSE NULL END FROM ${databaseNameAsId}.${ledgerTableNameAsId} "L"
              LEFT JOIN ${databaseNameAsId}.${trxTableNameAsId} "T" ON "L"."XaKey" = "T"."XaKey"
              WHERE "L"."EventSubscriber" = ${escapeStr(readModelName)}
              AND "L"."IsPaused" = FALSE
              AND "L"."Errors" IS NULL
              LIMIT 1
            );`);

          if ((rows === null || rows === void 0 ? void 0 : (_rows$ = rows[0]) === null || _rows$ === void 0 ? void 0 : _rows$.ActiveLocksCount) > 0) {
            isActive = true;
          }
        } catch (error) {
          if (!(error instanceof PassthroughError) || !error.isRetryable) {
            throw error;
          }
        }
      }

      const nextCursor = await ((_eventstoreAdapter$ge = eventstoreAdapter.getCursorUntilEventTypes) === null || _eventstoreAdapter$ge === void 0 ? void 0 : _eventstoreAdapter$ge.call(eventstoreAdapter, ...cursorAndEventTypes));
      const hasNextEvents = nextCursor != null && nextCursor !== cursorAndEventTypes[0];
      const isAlive = isActive || !hasNextEvents;
      result = Object.assign(result, {
        isAlive
      });
    } else if (includeRuntimeStatus) {
      var _result2;

      const isAlive = ((_result2 = result) === null || _result2 === void 0 ? void 0 : _result2.status) === 'skip';
      result = Object.assign(result, {
        isAlive
      });
    }

    return result;
  } finally {
    pool.activePassthrough = false;
  }
};

export default status;
//# sourceMappingURL=status.js.map